
#include "FlightDemo.h"
#include "resource.h"
#include "f18.h"
#include <math.h>


//////////////////////////////////////////////////////////////////////////////
//
// _Module : This is needed to use ATL Windows
//
//////////////////////////////////////////////////////////////////////////////

#define grayR 0x1F0000
#define grayG 0x3F0000
#define grayB 0x1F0000

#define darkSkyR 0x00000
#define darkSkyG 0x00000
#define darkSkyB 0x0F0000

CComModule _Module;

float sin_look[ANGLE_360];
float cos_look[ANGLE_360];
float sphere_cancel[ANGLE_60];

float nearStepSize;
float farStepSize;

float viewAngle = 60.0f * 3.14159265f / 180.0f;

int numberOfSlices = 160; //120;  // number of slices (equals nr of rows to render)
int nearScreenRow = 239;   // bottom line on the screen for rendering first slice
int farScreenRow = nearScreenRow - numberOfSlices;

int screen_z = 0;   // z position (height pos) of the screen in the world, z=0 screen touches the world

int play_x = 1000;  // player's x,y,z coordinates
int play_y = 1000;
int play_z = 260; //210;
int play_ang = ANGLE_30; // player's look angle
int play_dist = 100;    // player's distance from screen
int mountain_scale = 85;// general height scaler

int last_top[DISP_Y];   // cache for previous row height, used to prevent drawing spans that are behind
						// a span that is more in front
int screenBufOffset[320];

TDistInfo distInfo[200];  
 
int tiledTexture[512*512];
unsigned char heightMap[512*512];  // 8 bit height map

int shadeStep;
int fader = 0;
  
boolean saveFile = false;
boolean right_down = false;
boolean left_down = false;
int speed = 18;

DWORD temp;

int      m_FPSCounter;      // FPS counter
uint32_t m_FPSTicks[16];    // Ticks for last 16 frames

F18 *f18;

FlightDemo::FlightDemo()
{
    m_config.szWindowName     = _T("Texture Sample");
	m_config.parameters.m_orientation = ORIENTATION_EAST;
	m_config.parameters.m_swapEffect = SWAP_NONE;
}
 

FlightDemo::~FlightDemo() { 
}

bool FlightDemo::GameInit()
{
	m_display = GetDisplay();

	m_numbers = LoadImage(IDB_NUMBERS );
    m_FPSCounter = 0;
	HeightData = ::LoadImage(IDB_HEIGHT);
	HeightBuffer = HeightData->GetPixels();
	TextureData = LoadImage(IDB_TEXTURE);
	TextureBuffer = TextureData->GetPixels();
	InitTables();
	InitHeightMap();
	blitter = new Blitter();
	f18 = new F18(blitter);
	f18->InitF18(DISP_Y/2-100/2,100,0,0,0,true);
	return true;
}

// convert 565 RGB bitmap to 8 bit height map
void FlightDemo::InitHeightMap() {
	int min = 0xFFFFFF, max = 0;
	int u,v;
	Pixel *buffer = HeightBuffer;
	for (int i=0; i<512; i++) {
		for (int j=0; j<512; j++) {
			if (*buffer > max) max = *buffer;
			if (*buffer < min) min = *buffer;
			buffer++;
		}
	}
	float scale = (float)(max - min) / (float)255;
	buffer = HeightBuffer;
	for (u=0; u<512; u++) {
		for (int v=0; v<512; v++) {
			float scaledVal = ((float)(buffer[v*512+u]) - (float)min)/(float)scale;
			int dst = (u & 0xF) + ((u & 0x1F0) << 9) + (v << 4);
			heightMap[dst]=(unsigned char)scaledVal;
		}
	}

	for (u=0; u<512; u++) {
		for (v=0; v<512; v++) {
			int dst = (u & 0xF) + ((u & 0x1F0) << 9) + (v << 4);
			tiledTexture[dst] = TextureBuffer[v*512+u] + (heightMap[dst] << 16);
		}
	}
 
}

void FlightDemo::InitTables() {
	for (int ang=0; ang<ANGLE_360; ang++) {
		float rad_angle = (float)ang*ANGULAR_INCREMENT*DEG_TO_RAD;
		sin_look[ang] = (float)sin(rad_angle);
		cos_look[ang] = (float)cos(rad_angle);
		if (ang < ANGLE_30) { // fish eye cancel factor, not used for the moment
			sphere_cancel[ang+ANGLE_30] = 1/(float)cos(rad_angle);
			sphere_cancel[ANGLE_30-ang] = 1/(float)cos(rad_angle);
		}
	}
	// raylength is the distance from the player up to the position where the ray touches the world for
	// a specific slice (cfr a specific screen row, one slice corresponds to one screen row).
	// when rendering screen row r, the ray is the line from the player through the screen row, up to the floor on the world map
	for (int i=0; i<numberOfSlices; i++) {
		float delta = (float)play_z - (screen_z + (320 - nearScreenRow) + i); 
		distInfo[i].rayLength = ((float)(play_dist*play_z)/delta);
		distInfo[i].numRays = DISP_Y << 16;
		distInfo[i].lineThickness = 1;
	}
	// calculate stepsizes along the outer left and outer right rays
	// outer left & right are angled at viewAngle/2 from the player's view vector
	farStepSize = (distInfo[numberOfSlices-1].rayLength - distInfo[numberOfSlices-2].rayLength)/(float)cos(viewAngle/2.0f);
	nearStepSize = (distInfo[1].rayLength - distInfo[0].rayLength)/(float)cos(viewAngle/2.0f);
	float stepInc = (farStepSize - nearStepSize) / (float)(numberOfSlices - 1);
	float stepSize = nearStepSize;
	for (i=0; i<numberOfSlices-1; i++) {
		distInfo[i].stepSize = (long)((65636.0 * (distInfo[i+1].rayLength - distInfo[i].rayLength))/(float)cos(viewAngle/2.0f));
		//distInfo[i].stepSize = (long)(stepSize * 65536.0);
		stepSize += stepInc;
	}
	distInfo[numberOfSlices-1].stepSize =  0;

	for (i=0; i<numberOfSlices; i++) {
		for (int j=0; j<256; j++) 
			distInfo[i].perspectiveLookup[j] = (int)mountain_scale * j / (int)(distInfo[i].rayLength+1);
	}
	int offset = 0;
	for (i=0; i<DISP_X; i++) {
		screenBufOffset[i] = offset;
		offset += DISP_Y;
	}
	shadeStep = FPDIV(1<<16,(numberOfSlices*3/4)<<16);

}

void FlightDemo::DrawTerrain(int play_x,int play_y,int play_z,int play_ang,int play_dist, Pixel *buffer, Pixel *texture, unsigned char *height) {
	int left_ang,right_ang,
		y_fine,x_fine,
		scale,
		top,
		diff,i,offset,pix;
	Pixel *scroffset;
	Pixel tPix;
	unsigned char hPix; // 8 bit height value
	long fx,fy; // world coordinates for height & texture
	long ray_length;
	long ulx,uly,urx,ury; // world vectors ul and ur for edge stepping, these are unit vectors
	long wlx,wly,wrx,wry; // the cumulated vectors along the left/right edge
	long dx,dy;
	long invNumRays,stepSize,numRays,lineThickness;
	int maxDiff;
	TDistInfo *info; 
	int shader = 1<<16;

	left_ang = play_ang - ANGLE_30;
	if (left_ang < 0) left_ang += ANGLE_360;
	right_ang = play_ang + ANGLE_30;
	if (right_ang > ANGLE_360) right_ang -= ANGLE_360;

	ulx = (long)(cos_look[left_ang] * 65536.0);
	uly = (long)(sin_look[left_ang] * 65536.0);
	urx = (long)(cos_look[right_ang] * 65536.0);
	ury = (long)(sin_look[right_ang] * 65536.0);

	ray_length = (long)(distInfo[0].rayLength * 65536.0);

	wlx = (play_x << 16) + FPMUL(ulx,ray_length);
	wly = (play_y << 16) + FPMUL(uly,ray_length);
	wrx = (play_x << 16) + FPMUL(urx,ray_length);
	wry = (play_y << 16) + FPMUL(ury,ray_length);

	for(i=0; i<DISP_Y; i++) last_top[i] = 0xFFFF;  // optimize with memset

	for (int slice=0; slice<numberOfSlices; slice++) {
		info = &distInfo[slice];
				
		invNumRays = FPDIV(0x10000,info->numRays);
		dx = FPMUL(wrx-wlx,invNumRays);
		dy = FPMUL(wry-wly,invNumRays);
		fx = wlx;
		fy = wly; 

		if (slice > (numberOfSlices - (numberOfSlices * 3/4))) shader -= shadeStep; 

		lineThickness = info->lineThickness;
		numRays = info->numRays >> 16;

     	for (int col=0; col < numRays; col++) {

			int x_t = fx >> 2; // 1 texture coordinate corresponds with 4 world coordinates
			int y_t = fy >> 2;

			int u_frac = x_t & 0xFFFF;
			int v_frac = y_t & 0xFFFF;

			x_fine = (x_t >> 16) & 0x1FF;
			int x_fine1 = (x_fine + 1) & 0x1FF;
			y_fine = (y_t >> 16) & 0x1FF;
			int y_fine1 = (y_fine + 1) & 0x1FF;

			int offset0 = (x_fine & 0xF) | ((x_fine & 0x1F0) << 9) | (y_fine << 4);
			int offset1 = (x_fine1 & 0xF) | ((x_fine1 & 0x1F0) << 9) | (y_fine << 4);
			int offset2 = (x_fine1 & 0xF) | ((x_fine1 & 0x1F0) << 9) | (y_fine1 << 4);
			int offset3 = (x_fine & 0xF) | ((x_fine & 0x1F0) << 9) | (y_fine1 << 4);
			
			int pix0 = tiledTexture[offset0];
			int pix1 = tiledTexture[offset1];
			int pix2 = tiledTexture[offset2];
			int pix3 = tiledTexture[offset3];

			Pixel tPix0 = pix0 & 0xFFFF;
			Pixel tPix1 = pix1 & 0xFFFF;
			Pixel tPix2 = pix2 & 0xFFFF;
			Pixel tPix3 = pix3 & 0xFFFF;

			int f0 = FPMUL((1 << 16) - u_frac, (1<<16) - v_frac);
			int f1 = FPMUL(u_frac, (1<<16) - v_frac);
			int f2 = FPMUL(u_frac, v_frac);
			int f3 = FPMUL((1 << 16) - u_frac, v_frac);


			int r = FPMUL((tPix0 & RED_MASK) << 5, f0) +
					FPMUL((tPix1 & RED_MASK) << 5, f1) +
					FPMUL((tPix2 & RED_MASK) << 5, f2) +
					FPMUL((tPix3 & RED_MASK) << 5, f3);
	 		int g = FPMUL((tPix0 & GREEN_MASK) << 11, f0) +
					FPMUL((tPix1 & GREEN_MASK) << 11, f1) +
					FPMUL((tPix2 & GREEN_MASK) << 11, f2) +
					FPMUL((tPix3 & GREEN_MASK) << 11, f3);
			int b = FPMUL((tPix0 & BLUE_MASK) << 16, f0) +
					FPMUL((tPix1 & BLUE_MASK) << 16, f1) +
					FPMUL((tPix2 & BLUE_MASK) << 16, f2) +
					FPMUL((tPix3 & BLUE_MASK) << 16, f3);

			r = FPMUL(FPMUL(r,fader),shader) + FPMUL(0x10000 - shader, grayR);
			g = FPMUL(FPMUL(g,fader),shader) + FPMUL(0x10000 - shader, grayG);
			b = FPMUL(FPMUL(b,fader),shader) + FPMUL(0x10000 - shader, grayB);

			tPix = ((r >> 16) << RED_SHIFT) +
				   ((g >> 16) << GREEN_SHIFT) +
				   (b >> 16) ;


			hPix = (FPMUL(pix0 & 0xFF0000, f0) +
				   FPMUL(pix1 & 0xFF0000, f1) +
				   FPMUL(pix2 & 0xFF0000, f2) +
				   FPMUL(pix3 & 0xFF0000, f3)) >> 16;
 
			scale = info->perspectiveLookup[hPix];

			top = nearScreenRow-slice-scale;
			diff = last_top[col] - top;
			if (diff>0) {
				maxDiff = nearScreenRow - top;
				if (diff > maxDiff) diff = maxDiff;
				scroffset = buffer + screenBufOffset[top] + col;
				for (i=diff; i; i--) {
					*scroffset = (int)tPix;
					scroffset += DISP_Y;
				}
				last_top[col] = top;
			}
			fx += dx;
			fy += dy;
		}

		stepSize = info->stepSize;
		wlx += FPMUL(stepSize,ulx);
		wly += FPMUL(stepSize,uly);
		wrx += FPMUL(stepSize,urx);
		wry += FPMUL(stepSize,ury);
	  
	}
}
 

void FlightDemo::GameLoop() 
{

	Surface *BackSurface = m_display->GetBackBuffer();
	Pixel* BackBuffer = BackSurface->GetPixels();

	if (right_down) {
		f18->SetFrame(2); 
		play_ang += speed;
	}
	else if (left_down) {
		f18->SetFrame(1); 
		play_ang -= speed;
	}
	else f18->SetFrame(0);

	if (play_ang > ANGLE_360) play_ang -= ANGLE_360;
	else if (play_ang < ANGLE_0) play_ang += ANGLE_360;

	play_x += (int)(cos_look[play_ang]*speed);
	play_y += (int)(sin_look[play_ang]*speed);

	Pixel *dbuf = BackBuffer;
	Pixel *dptr = dbuf;
	int alpha = 0x10000;
	int alphaStep = FPDIV(alpha,80<<16);
	for (int i=0; i<80; i++) {
		int r = FPMUL(0x10000-alpha,grayR) + FPMUL(alpha,darkSkyR);
		int g = FPMUL(0x10000-alpha,grayG) + FPMUL(alpha,darkSkyG);
		int b = FPMUL(0x10000-alpha,grayB) + FPMUL(alpha,darkSkyB);
		for (int j=0; j<DISP_Y; j++) {
			*dptr++ = ((r>>16) << RED_SHIFT) + ((g>>16) << GREEN_SHIFT) + (b>>16);
		}
		alpha-=alphaStep;
	}


	DrawTerrain(play_x,play_y,play_z,play_ang,play_dist,BackBuffer,TextureBuffer,heightMap);

	// Update FPS
    m_FPSTicks[ m_FPSCounter & 15 ] = GetTickCount();
    if (m_FPSCounter > 15)
    {
        uint32_t totalTime = m_FPSTicks[ m_FPSCounter & 15 ] - m_FPSTicks[ (m_FPSCounter+1) & 15 ];
        if (totalTime == 0) totalTime = 1;
        uint32_t fps  = 16000 / totalTime;
        uint32_t n100 = fps / 100; fps -= n100 * 100;
        uint32_t n10  = fps / 10;  fps -= n10 * 10;
        uint32_t n1   = fps;

        if (n100 > 9) { n100 = 9; n10 = 9; n1 = 9; }
        
        // Display fps
        blitter->BlitOn( m_numbers, &Rect( n100 * 8, 0, (n100+1)*8, 11 ), BackSurface, 0, 0, 0 );
        blitter->BlitOn( m_numbers, &Rect( n10 * 8, 0, (n10+1)*8, 11 ), BackSurface,  8, 0, 0 );
        blitter->BlitOn( m_numbers, &Rect( n1 * 8, 0, (n1+1)*8, 11 ), BackSurface, 16, 0, 0 );
    }
    ++m_FPSCounter;
	

	if (fader < (0xA000)) fader += 1024;
	else f18->Draw(BackSurface); 

	if (saveFile) SaveImage(BackSurface,_T("/Temp/screen.bmp"));

    m_display->Swap();
	
}


void FlightDemo::GameEnd()
{
	delete HeightData;
	delete TextureData;
	delete f18;
	delete blitter; 
}


void FlightDemo::ButtonDown( int button )
{

   if (button > 1 )
   {
	   KeyList *keys = new KeyList();
	   ::GetKeyList(keys,ORIENTATION_NORTH);

	    if(button == keys->vkA) 
		{
			saveFile = true;
		}
		if(button == VK_LEFT)
		{
			left_down = true;
		}
		if(button == VK_RIGHT)
		{
			right_down = true;
		}
		if(button == VK_UP)
		{
			speed += 1;
			if (speed > 100) speed = 100;
		}
		if(button == VK_DOWN)
		{
			speed -= 1;
		}	if (speed < 0) speed = 0;
		delete keys;

   }//if button > 1 end
}//end button down


void FlightDemo::ButtonUp( int button )
{
   if (button > 1 )
   {
	   KeyList *keys = new KeyList();
	   ::GetKeyList(keys,ORIENTATION_NORTH);

   	    if(button == keys->vkA) 
		{
			saveFile = false;
		}
		if(button == VK_LEFT)
		{
			left_down = false;
		}
		if(button == VK_RIGHT)
		{
			right_down = false;
		}

		delete keys;

   }//if button > 1 end
}

 
//////////////////////////////////////////////////////////////////////////////
//
// WinMain - Entry point
//
//////////////////////////////////////////////////////////////////////////////

int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE, LPTSTR, int )
{
    _Module.Init( 0, hInstance );
    
    FlightDemo game;
    
	game.Run();
    
    return 0;
}
